unit BrowseDlg;

interface

uses
  PlatformStuff, WriteDlg, Threading,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.ListBox, FMX.Layouts, FMX.Objects, FMX.ListView.Types,
  FMX.ListView, ServerData,dOPCIntf, FMX.Menus,ItemPropDlg,
  FMX.ListView.Appearances, FMX.ListView.Adapters.Base,
  FMX.Controls.Presentation, FMX.MultiView;

type
  TBrowseForm = class(TCoreForm)
    ListView: TListView;
    PopupMenu1: TPopupMenu;
    MenuOk: TMenuItem;
    MenuCancel: TMenuItem;
    MenuProp: TMenuItem;
    MenuItemWrite: TMenuItem;
    Panel1: TPanel;
    LPath: TLabel;
    bUp: TButton;
    bDown: TButton;
    MenuUp: TMenuItem;
    MenuDown: TMenuItem;
    bProp: TSpeedButton;
    bWrite: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MenuCancelClick(Sender: TObject);
    procedure MenuOkClick(Sender: TObject);
    procedure MenuPropClick(Sender: TObject);
    procedure MenuItemWriteClick(Sender: TObject);
    procedure MenuDownClick(Sender: TObject);
    procedure MenuUpClick(Sender: TObject);
  private
    BMPFolder  : TBitmap;
    BMPItem    : TBitmap;
    BMPIFolder : TBitmap;
    OPCServer  : TOPCDAServer;
    fInProcess : boolean;
    fCanSelect : boolean;

    procedure ShowLevel;
    procedure MoveUp;
    procedure MoveDown;
  public
    OPCItemIds : TStringlist;
    procedure ShowBrowseDlg(SItem: TServerItem; CanSelect: boolean);
  end;

var
  BrowseForm: TBrowseForm;


implementation

{$R *.fmx}


procedure TBrowseForm.ShowBrowseDlg(SItem: TServerItem; CanSelect: boolean);
begin
  OPCServer := SItem.FServer;
  OPCItemIds.Clear;
  fCanSelect := CanSelect;
  ShowMenuItem(MenuOk,CanSelect);
  Listview.EditMode := CanSelect;
  ShowLevel;
end;

procedure TBrowseForm.ShowLevel;
var
  BItem : TdOPCBrowseItem;
  LItem : TListViewItem;
begin

   if not OPCServer.active then
   begin
     Listview.EditMode := false;
     Listview.Items.Clear;
     LItem := Listview.Items.Add;
     LItem.Text := 'not connected';
     exit;
   end;
   LPath.Text := 'Root'+OPCServer.Browser.GetCurrentPath;
   OPCServer.Browser.AccessRights :=[];
   Listview.Items.Clear;
   Listview.BeginUpdate;
   try
     Listview.ItemIndex := -1;
     Listview.Selected := nil;
     OPCServer.Browser.Browse;
     for BItem in OPCServer.Browser.Items do
     begin
       LItem :=  Listview.Items.Add;
       LItem.Tag  := NativeInt(BItem);
       LItem.Text := BItem.Name;
       if BItem.IsFolder then
       begin
        if BItem.IsItem then
          LItem.BitmapRef := BMPIFolder
        else
          LItem.BitmapRef := BMPFolder
       end
       else
         LItem.BitmapRef := BMPItem;
     end;
     if Listview.Items.Count > 0 then
       Listview.Selected :=  Listview.Items[0];

   finally
     Listview.EndUpdate;
   end;
end;


procedure TBrowseForm.MoveUp;
begin
  if not OPCServer.active then
     exit;
  if OPCServer.Browser.CurrentPosition.Name <> '' then // if not root folder
  begin
    OPCServer.Browser.Moveup;
    ShowLevel;
  end;
end;

(*
function GetBrowseItemData(ListView: TListView): TdOPCBrowseItem;
var
  LItem : TListViewItem;
begin
  result := nil;
  LItem := TListViewItem(Listview.Selected);
  if LItem <> nil then
    result := TdOPCBrowseItem(LItem.Tag);
end;
*)
function GetBrowseItemData(ListView: TListView): TdOPCBrowseItem;
var
  LItem : TListItem;
begin
  result := nil;
  LItem :=Listview.Selected;
  if LItem <> nil then
    result := TdOPCBrowseItem(LItem.Tag);
end;


procedure TBrowseForm.MoveDown;
var
 BrowseItem : TdOPCBrowseItem;
begin
  BrowseItem := GetBrowseItemData(ListView);
  if not OPCServer.active  or  (BrowseItem = nil) then
     exit;
  if Browseitem.IsFolder then   // if branch
  begin
     OPCServer.Browser.MoveDown(Browseitem);
     ShowLevel;
  end;
end;

procedure TBrowseForm.FormCreate(Sender: TObject);
begin
  inherited;
  OPCItemIds := TStringlist.Create;
  BMPFolder  := CreateBMPFromResource('FOLDER');
  BMPItem    := CreateBMPFromResource('ITEM');
  BMPIFolder := CreateBMPFromResource('FOLDERITEM');
  AddPopup('Browse Items',PopupMenu1);
  fInProcess := false;
end;

procedure TBrowseForm.FormDestroy(Sender: TObject);
begin
  inherited;
  BMPFolder.DisposeOf;
  BMPItem.DisposeOf;
  BMPIFolder.DisposeOf;
  OPCItemIds.Free;
end;

procedure TBrowseForm.MenuCancelClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TBrowseForm.MenuDownClick(Sender: TObject);
begin
  MoveDown;
end;

procedure TBrowseForm.MenuUpClick(Sender: TObject);
begin
  MoveUp;
end;


procedure TBrowseForm.MenuOkClick(Sender: TObject);
var
  LItem: TListviewItem;
  BItem: TdOPCBrowseItem;
begin
  OPCItemIds.Clear;
  for LItem in Listview.Items do
  begin
    BItem := TdOPCBrowseItem(LItem.Tag);
    if LItem.Checked and BItem.IsItem then
       OPCItemIds.Add(BItem.ItemId);
  end;
 BackToParentForm(true);
end;

procedure TBrowseForm.MenuPropClick(Sender: TObject);
var
  BItem : TdOPCBrowseItem;
begin
  BItem := GetBrowseItemData(Listview);
  if BItem <> nil then
  begin
    ShowNewForm(ItemPropForm,TItemPropForm);
    ItemPropForm.ShowProperties(OPCServer, BItem.ItemId);
  end;
end;


procedure TBrowseForm.MenuItemWriteClick(Sender: TObject);
var
  BItem : TdOPCBrowseItem;
begin
  BItem := GetBrowseItemData(Listview);
  if BItem <> nil then
  begin
    ShowNewForm(WriteForm,TWriteForm);
    WriteForm.WriteItem(OPCServer, nil, BItem.ItemId);
  end;
end;

end.
