unit CoreDlg;

interface
uses
  FMX.Platform, Sysutils, PlatformStuff, WaitDlg,
  System.IOUtils,System.Types, System.Classes,  System.UITypes,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.ListBox, FMX.Objects, System.Actions, FMX.ActnList, FMX.Menus, FMX.Layouts,
  FMX.Controls.Presentation, FMX.MultiView, FMX.DialogService;

type
  TYesAction = reference to procedure;

  TCoreForm = class(TForm)
    Header: TToolBar;
    Footer: TToolBar;
    HeaderLabel: TLabel;
    bInfo: TButton;
    bMasterButton: TButton;
    MultiView1: TMultiView;
    ListItemBack: TListBoxItem;
    ListBoxHeader1: TListBoxHeader;
    MenuCaption: TLabel;
    bBack: TButton;
    ListMenu: TListBox;
    procedure bInfoClick(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure FormVirtualKeyboardHidden(Sender: TObject; KeyboardVisible: Boolean; const Bounds: TRect);
    procedure FormVirtualKeyboardShown(Sender: TObject;  KeyboardVisible: Boolean; const Bounds: TRect);
    procedure ListItemBackClick(Sender: TObject);
    procedure ListMenu2ItemClick(const Sender: TCustomListBox; const Item: TListBoxItem);
    procedure bBackClick(Sender: TObject);
    procedure FormResize(Sender: TObject);

  private
    fKeyboardVisible : boolean;
    fOnOk      : TNotifyEvent;
    fMenuName  : string;
    procedure ShowMenu(LItem: TComponent);



  protected
    ParentForm : TCoreForm;
    procedure HideMenu;
    procedure OnItemClick(Sender: TObject);
    // procedure Show_Menu(MenuId: integer);
  public
    procedure ShowNewForm(var Form; const InstanceClass: TComponentClass ; OnOk: TNotifyEvent = nil);
    procedure BackToParentForm(Ok: boolean = false); virtual;
    procedure AddPopup(MenuName: string; Popup: TPopupMenu);
    procedure _WaitFor(Info: string; AThreadProc: TThreadProcedure);
    procedure ShowWaitForm(Info:string);
    procedure HideWaitForm;
    procedure ShowMenuItem(MenuItem: TComponent; Show: boolean);
  end;

  procedure IfYes(Question: string; YesAction: TYesAction);


{$IFNDEF NEXTGEN}
  {$DEFINE DESKTOP}
{$ENDIF}

implementation

{$R *.fmx}

// a wrapper for the Yes/No dialog
procedure IfYes(Question: string; YesAction: TYesAction);
begin
  FMX.DialogService.TDialogService.MessageDialog(
    Question, System.UITypes.TMsgDlgType.mtInformation,
    [ System.UITypes.TMsgDlgBtn.mbYes,
      System.UITypes.TMsgDlgBtn.mbNo], System.UITypes.TMsgDlgBtn.mbYes,0,
      procedure(const AResult: System.UITypes.TModalResult)
      begin
        case AResult of
          mrYES:
            YesAction;
        end;
      end);
end;


procedure TCoreForm.ShowMenuItem(MenuItem: TComponent; Show: boolean);
var
  i : integer;
  Item: TListBoxItem;
begin
 for i := 0 to ListMenu.Count-1 do
 begin
    Item := ListMenu.ItemByIndex(i);
    if Item.Data = MenuItem then
    begin
      Item.Visible := Show;
      break;
    end;
  end;
end;


procedure TCoreForm.ShowMenu(LItem: TComponent);
var
  i : integer;
  Item  : TListBoxItem;
begin
  try
   ListMenu.BeginUpdate;
   for i := 0 to ListMenu.Count-1 do
   begin
    Item := ListMenu.ItemByIndex(i);
    Item.Visible :=  TListBoxItem(Item.Tag) = LItem;
    Item.Enabled :=  Item.Visible;
   end;
  finally
    ListMenu.EndUpdate;
  end;
end;


// hide menu if calling a new menu
procedure TCoreForm.OnItemClick(Sender: TObject);
var
  LItem  : TListBoxItem;
begin
  if Sender is TListBoxItem then
  begin
    LItem := TListBoxItem(Sender);
    if assigned(TMenuItem(LItem.Data).OnClick) then
    begin
      HideMenu;
      Application.ProcessMessages;
      TMenuItem(LItem.Data).OnClick(Sender);
      //Application.ProcessMessages;
      //TThread.Queue(TThread.Current,procedure() begin TMenuItem(LItem.Data).OnClick(Sender); end );
      //TTask.Create(Self,TMenuItem(LItem.Data).OnClick).
    end
    else // sub menu expected
    begin
      if LItem.ItemData.Accessory = TListBoxItemData.TAccessory.aMore then
        ShowMenu(LItem)
      else
        ShowMenu(nil)
    end;
  end;
end;

// adds all menu item to the listbox
procedure TCoreForm.AddPopup(MenuName: string; Popup: TPopupMenu);

function CreateListItem(MItem : TMenuItem; ParentMenu: TListBoxItem; Visible: boolean): TListBoxItem;
begin
  result            := TListBoxItem.Create(nil);
  result.Data       := MItem;
  result.Text       := MItem.Text;
  result.Height     := 38;
  result.OnClick    := OnItemClick;
  result.Visible    := Visible;
  result.IsChecked  := false;
  result.IsSelected := false;
  result.Tag        := integer(ParentMenu);
  if MItem.ItemsCount > 0 then
    result.ItemData.Accessory := TListBoxItemData.TAccessory.aMore;
  ListMenu.AddObject(result);
end;

procedure CreateSubMenu(MItem : TMenuItem; ParentMenu: TListBoxItem);
var
  i : integer;
begin
 for i := 0 to MItem.ItemsCount-1 do
 begin
   CreateListItem(MItem.Items[i],ParentMenu,false);
   if MItem.Items[i].ItemsCount > 0 then
     CreateSubMenu(MItem.Items[i],ParentMenu)
 end;
end;

var
  i: Integer;
  MItem : TMenuItem;
  // MenuId: integer;
  MenuItem: TListBoxItem;
begin
  if fMenuName = MenuName then
    exit;
  fMenuName := MenuName;
  try
    ListMenu.BeginUpdate;
    MenuCaption.Text := fMenuName;
    ListMenu.Clear;
//    MenuId := 0;
    for i := 0 to Popup.ItemsCount-1 do
    begin
      MItem := Popup.Items[i];
      if MItem.Visible then
      begin
        MenuItem := CreateListItem(MItem,nil,true);
        if MItem.ItemsCount > 0 then
        begin
          CreateSubMenu(MItem,MenuItem)
        end;
      end;
    end;
  finally
   ListMenu.EndUpdate;
  end;
end;

procedure TCoreForm.HideMenu;
begin
   MultiView1.DrawerOptions.DurationSliding := 0;
   MultiView1.HideMaster;
   MultiView1.DrawerOptions.DurationSliding := 0.25;
   ListMenu.ClearSelection;
end;



procedure TCoreForm.ListItemBackClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TCoreForm.ListMenu2ItemClick(const Sender: TCustomListBox;
  const Item: TListBoxItem);
begin
  Item.IsSelected := false;
end;

procedure TCoreForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  {$IFDEF DESKTOP}
  BackToParentForm;
  {$ENDIF}
end;

procedure TCoreForm.FormCreate(Sender: TObject);
begin
  fKeyboardVisible   := false;
  fMenuName          := '';
  fOnOk              := nil;
  ParentForm         := nil;
end;

procedure TCoreForm.FormKeyUp(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
begin
  if (Key = vkHardwareBack) or (Key = vkEscape) then
  begin
    if not fKeyboardVisible then
    begin
      if not MultiView1.IsShowed then
      begin
        if ParentForm = nil then
          IfYes('Exit Program ?',procedure begin ExitApp; end);
        BackToParentForm;
      end
      else HideMenu;
      Key     := 0;
      KeyChar := #0;
    end
  end
  else
    if (Key = vkMenu) then
    begin
       if MultiView1.IsShowed then
         HideMenu
       else
         MultiView1.ShowMaster
    end;
end;

procedure TCoreForm.FormResize(Sender: TObject);
begin
  if Width > 700 then
    MultiView1.Mode :=  TMultiViewMode.Panel
  else
    MultiView1.Mode :=  TMultiViewMode.Drawer;
end;

procedure TCoreForm.FormVirtualKeyboardHidden(Sender: TObject;
  KeyboardVisible: Boolean; const Bounds: TRect);
begin
  fKeyboardVisible := false;
end;

procedure TCoreForm.FormVirtualKeyboardShown(Sender: TObject;
  KeyboardVisible: Boolean; const Bounds: TRect);
begin
  fKeyboardVisible := true;
end;

procedure TCoreForm.BackToParentForm(Ok: boolean);
begin
  HideMenu;
  if ParentForm <> nil then
  begin
    if OK and Assigned(ParentForm.fOnOk) then
      ParentForm.fOnOK(self);
    ShowNewForm(ParentForm,nil);
  end
end;

procedure TCoreForm.bInfoClick(Sender: TObject);
begin
  ShowMessage('dOPC Mobile Explorer'+ LineFeed+
              '(C) 2022 Kassl GmbH '+ LineFeed+
              'http://www.kassl.de');
end;

procedure TCoreForm.bBackClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TCoreForm.ShowNewForm(var Form; const InstanceClass: TComponentClass; OnOk: TNotifyEvent);
begin
  if not assigned(TCoreForm(Form)) then
  begin
    if InstanceClass = nil then
      raise Exception.Create('ERROR form instance is NIL');
    Application.CreateForm(InstanceClass,Form);
  end;
  fOnOk := OnOk;

  if ParentForm <> TCoreForm(Form) then
    TCoreForm(Form).ParentForm := self;
  if not assigned(TCoreForm(Form).StyleBook) then
    TCoreForm(Form).StyleBook := StyleBook;

 {$IFDEF DESKTOP}
    Application.ProcessMessages;
    TCoreForm(Form).SetBounds(Left,Top,Width,Height);
    Application.ProcessMessages;
    TCoreForm(Form).Show;
    Hide;
  {$ELSE}
    TCoreForm(Form).Show;
  {$ENDIF}
end;

procedure TCoreForm.HideWaitForm;
begin
  if WaitFrame <> nil then
    WaitFrame.HideWait;
end;
procedure TCoreForm.ShowWaitForm(Info: string);
begin
  if not Assigned(WaitFrame) then
    WaitFrame := TWaitFrame.Create(Parent);
  WaitFrame.ShowWait(Self);
end;

procedure TCoreForm._WaitFor(Info: string; AThreadProc: TThreadProcedure);
begin
  TWaitFrame.RunWaitDlg(self,Info,AThreadProc);
end;

end.


