unit ServerData;

interface
uses
  classes, dOPCDA, dOPCIntf;

type

  TOPCDAServer = class(TdOPCDAClient)
  strict private
    FCaption: string;
    FUser: string;
    FProxy: string;
    FPassword: string;
  public
    constructor Create(Aowner: TComponent); override;
  published
    property Caption    : string read FCaption write FCaption;
    property Proxy      : string read FProxy write FProxy;
    property User       : string read FUser write FUser;
    property Password   : string read FPassword write FPassword;
  end;

  TOPCGroup = class
  end;

  TServerItem = class(TCollectionItem)
    FServer   : TOPCDAServer;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    property Server: TOPCDAServer read FServer write FServer;
  end;

  TOPCServerList = class(TCollection)
  private
    function  GetItem(Index: Integer): TServerItem;
    procedure SetItem(Index: Integer; const Value: TServerItem);
   public
    function  Add: TServerItem;
    function  DeleteServer(Item: TServerItem): boolean;
    property  Items[Index: Integer]: TServerItem read GetItem write SetItem; default;
  end;

  TOPCExplorerData = class(TComponent)
  private
    fwidth: integer;
    ftop: integer;
    fheight: integer;
    fleft: integer;
    fAutoConnect : boolean;
    fServerList: TOPCServerList;
    fStyleFile : string;
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream : TStream; C: TComponent);

  public
    const posUndef =  -99999;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveToFile(ToFile: string);
    procedure LoadFromFile(FromFile: string);
    procedure CloseAll;
    function IsOneServerConnected: boolean;
  published
    property left  : integer read fleft write fleft;
    property top   : integer read ftop write ftop;
    property width : integer read fwidth write fwidth;
    property height: integer read fheight write fheight;
    property AutoConnect: boolean read fAutoConnect write fAutoConnect;
    property ServerList : TOPCServerList read FServerList write FServerList;
    property StyleFile: string read fStyleFile write fStyleFile;
  end;

implementation

{ TOPCServer }

constructor TServerItem.create(Collection: TCollection);
begin
  inherited;
  FServer   := TOPCDAServer.Create(nil);
  FServer.OPCGroupDefault.IsActive := false;
  FServer.SetSubComponent(true);        // to store data to file
end;

destructor TServerItem.destroy;
begin
  FServer.Free;
  inherited;
end;

{ TServerData }

constructor TOPCDAServer.create;
begin
  inherited create(nil);
  Protocol := coUA;
  //FSName := '';
  FCaption:= '';
  FUser:= '';
  FProxy:= '';
  FPassword:= '';
end;

{ TOPCServerList }



function TOPCServerList.DeleteServer(Item: TServerItem): boolean;
var
  i : integer;
begin
  result := false;
  for i := 0 to Count-1 do
    if Items[i] = Item then
    begin
      Delete(i);
      result := true;
      break;
    end;
end;

function TOPCServerList.GetItem(Index: Integer): TServerItem;
begin
  Result := TServerItem(inherited GetItem(Index));
end;

procedure TOPCServerList.SetItem(Index: Integer; const Value: TServerItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TOPCExplorerData.SaveToStream(Stream : TStream; C: TComponent);
var
  BinStream:TMemoryStream;
begin
  BinStream := TMemoryStream.create;
  try
    BinStream.WriteComponent(C);
    BinStream.Seek(0, TSeekOrigin.soBeginning);
    ObjectBinaryToText(BinStream, Stream);
  finally
    BinStream.Free
  end;
end;


procedure TOPCExplorerData.LoadFromStream(Stream : TStream);
var
  BinStream: TMemoryStream;
begin
  BinStream := TMemoryStream.Create;
  try
    ObjectTextToBinary(Stream, BinStream);
    BinStream.Seek(0, TSeekOrigin.soBeginning);
    BinStream.ReadComponent(self);
  finally
    BinStream.Free;
  end;
end;


procedure TOPCExplorerData.LoadFromFile(FromFile: string);
var
  StrStream: TMemoryStream;
begin
  StrStream := TMemoryStream.create;
  StrStream.LoadfromFile(FromFile);
  StrStream.Seek(0, TSeekOrigin.soBeginning);
  try
    LoadFromStream(StrStream);
  finally
    StrStream.Free;
  end;
end;

procedure TOPCExplorerData.SaveToFile(ToFile: string);
var
  StrStream:TMemoryStream;
begin
  StrStream := TMemoryStream.create;
  try
    SaveToStream(StrStream,self);
    StrStream.Seek(0, TSeekOrigin.soBeginning);
    StrStream.SaveToFile(ToFile);
  finally
    StrStream.Free;
  end;
end;

procedure TOPCExplorerData.CloseAll;
var
  SItem : TServerItem;
  i     : integer;
  Group : TdOPCGroup;
begin
  try
    for i := 0 to Serverlist.Count-1 do
    begin
      SItem := TServerItem(Serverlist[i]);
      try
        if SItem.Server.Active then
          SItem.Server.Active := false;
        for Group in SItem.Server.OPCGroups do
          Group.IsActive := false;
      except
      end;
    end;
  except
  end;
end;

function TOPCExplorerData.IsOneServerConnected: boolean;
var
  SItem : TServerItem;
  i     : integer;
begin
  result := false;
  for i := 0 to Serverlist.Count-1 do
  begin
    SItem := TServerItem(Serverlist[i]);
      if SItem.Server.Active then
      begin
        result := true;
        break;
      end;
  end;
end;


constructor TOPCExplorerData.create(AOwner: TComponent);
begin
  inherited;
  fleft := posUndef;
  ftop  := posUndef;
  fServerList := TOPCServerList.Create(TServerItem);
  fAutoConnect := false;
  fStyleFile := '';
end;

destructor TOPCExplorerData.destroy;
begin
  FServerList.DisposeOf;
  inherited;
end;


function TOPCServerList.Add: TServerItem;
begin
  result := TServerItem(inherited Add);
end;

initialization
end.

