unit ServerStatusDlg;

interface

uses
  dopcda, dopccomn, FMX.Header,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  CoreDlg, FMX.ListBox, FMX.Layouts, FMX.Objects, FMX.Menus,
  FMX.ListView.Types, FMX.ListView, System.Rtti, FMX.Grid, ServerData,
  FMX.Controls.Presentation, FMX.MultiView, FMX.Grid.Style, FMX.ScrollBox;

type
  TServerStatusForm = class(TCoreForm)
    PopupMenu1: TPopupMenu;
    PropGrid: TStringGrid;
    ColProp: TStringColumn;
    ColVal: TStringColumn;
    MenuBack: TMenuItem;
    Dummy: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure MenuBackClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    fRow : integer;
    procedure AddProp(const Prop, Value: string);
  public
    procedure ShowProperties(SItem : TServerItem);
  end;

var
  ServerStatusForm: TServerStatusForm;

implementation

{$R *.fmx}

type
  TCustomGridHelper = class helper for TCustomGrid
  public
    procedure AdjustStretchColumn(ACol: Integer);
  end;


procedure TCustomGridHelper.AdjustStretchColumn(ACol: Integer);
var
  I: Integer;
  W: Single;
begin
  W := 5 + VScrollBar.Width;
  for I := 0 to ColumnCount - 1 do begin
    if I = ACol then Continue;
    if Columns[I].Visible then
      W := W + Columns[I].Width;
  end;
  Columns[ACol].Width := Width - W;
end;


procedure TServerStatusForm.FormCreate(Sender: TObject);
begin
  inherited;
  fRow := 0;
  AddPopup('Server Status',PopupMenu1);
//  PropGrid.StyledSettings := PropGrid.StyledSettings - [TStyledSetting.Size];
//  PropGrid.TextSettings.Font.Size := Dummy.TextSettings.Font.Size;
end;

procedure TServerStatusForm.FormResize(Sender: TObject);
begin
  inherited;
  PropGrid.Columns[1].Width := Width - PropGrid.Columns[0].Width;
end;

procedure TServerStatusForm.MenuBackClick(Sender: TObject);
begin
  BackToParentForm;
end;

procedure TServerStatusForm.AddProp(const Prop: string; const Value: string);
begin
  PropGrid.Cells[0,fRow] := Prop;
  PropGrid.Cells[1,fRow] := Value;
  Inc(fRow);
end;


procedure TServerStatusForm.ShowProperties(SItem : TServerItem);
var
 SState : tServerStateRec;
 Server : TdOPCDAClient;
begin
  HideMenu;
  fRow := 0;
  Server := SItem.Server;
  ColProp.BeginUpdate;
  ColVal.BeginUpdate;
  try

    AddProp('Name', SItem.Server.Caption);
    AddProp('URL',  Server.ServerName);

    if Server.Active then
    begin
      PropGrid.RowCount :=  9;
      ShowWaitForm('Get Server Status...');
      try
        Server.GetServerInfo(SState);
        AddProp('Vendor info', SState.VendorInfo);
        AddProp('Version', SState.Version);
        AddProp('Type', Server.ServerTypeName);
        AddProp('State', ServerStateToStr(SState.ServerState,Server.Protocol));
        AddProp('State info',  SState.StatusInfo);
        AddProp('Start time',  DateTimeToStr(SState.Starttime));
        AddProp('Update time', DateTimeToStr(SState.LastUpdateTime));
      finally
        HideWaitForm;
      end;
    end
    else
      AddProp('State','not connected');
    PropGrid.RowCount := fRow;
    PropGrid.Columns[1].Width := Width - PropGrid.Columns[0].Width;
    //PropGrid.AdjustStretchColumn(1);
  finally
    ColProp.EndUpdate;
    ColVal.EndUpdate;
  end;
end;


end.
